function [U, V, centroidV, Log, Beta, ac] = HINMF(X, K, label, options)
% This is a module of Heterogeneous Information Network Non-negative Matrix Factorization (HINMF)
%
% where
%   X
% Notation:
% X ... a cell containing all views for the data
% K ... number of hidden factors
% label ... ground truth labels
% Written by Jialu Liu (jliu64@illinois.edu)

viewNum = length(X);
Rounds = options.rounds;
U_ = [];
V_ = [];

U = cell(1, viewNum);
V = cell(1, viewNum);

RE = zeros(1, viewNum);
beta = zeros(1, viewNum);

j = 0;
Log = 0;
Beta =[0;0];
ac = 0;

        
% initialize basis and coefficient matrices
while j < 1
    j = j + 1;
    if j == 1
        [U{1}, V{1}] = NMF(X{1}, K, options, [], []);
        printResult(V{1}, label, K, options.kmeans);
    else
        [U{1}, V{1}] = NMF(X{1}, K, options, [], []);
        printResult(V{1}, label, K, options.kmeans);        
    end
    for i = 2:viewNum
        [U{i}, V{i}] = NMF(X{i}, K, options, [], []);
        printResult(V{i}, label, K, options.kmeans);
    end
end

centroidV = V{1};
for i=1:viewNum
    tmp1 = X{i} - U{i}*V{i}';
    tmp2 = V{i} - centroidV;
    RE(i) = sum(sum(tmp1.^2)) + options.alpha * sum(sum(tmp2.^2));
end

for i=1:viewNum
    beta(i) = -log(RE(i) / sum(RE));
end
Beta(:,end+1) = [beta(1); beta(2)];
optionsForPerViewNMF = options;
oldL = 100;

tic
j = 0;
while j < Rounds
    j = j + 1;
    if j==1
        centroidV = V{1};
    else
        centroidV = beta(1) * V{1};
        for i = 2:viewNum
            centroidV = centroidV + beta(i) * V{i};
        end
        
        centroidV = centroidV / sum(beta);
        for i=1:viewNum
            tmp1 = X{i} - U{i}*V{i}';
            tmp2 = V{i} - centroidV;
            RE(i) = sum(sum(tmp1.^2)) + options.alpha * sum(sum(tmp2.^2));
        end

        for i=1:viewNum
            beta(i) = -log(RE(i) / sum(RE));
        end
        Beta(:,end+1) = [beta(1); beta(2)];
    end
    %sum(options.alpha)
    logL = 0;
    for i = 1:viewNum
        logL = logL + beta(i)*RE(i);
    end
    Log(end+1) = logL;
    logL
    if(oldL < logL)
        U = oldU;
        V = oldV;
        logL = oldL;
        j = j - 1;
        disp('restrart this iteration');
    else
        ac(end+1) = printResult(centroidV, label, K, options.kmeans);
    end
    
    oldU = U;
    oldV = V;
    oldL = logL;
    
    for i = 1:viewNum
        optionsForPerViewNMF.alpha = options.alpha;
        [U{i}, V{i}] = PerViewNMF(X{i}, K, centroidV, optionsForPerViewNMF, U{i}, V{i});
    end
end
toc